#ifndef _TCFS_H_
#define _TCFS_H_

#ifdef __cplusplus
extern "C" {
#endif

enum{
    TCFS_ERRCODE_OK = 0,        
    TCFS_ERRCODE_INITIALIZE_ERROR,    
    TCFS_ERRCODE_ALREADY_INITIALIZED,    
    TCFS_ERRCODE_UNINITIALIZED,
    TCFS_ERRCODE_PATH_ERROR,            
    TCFS_ERRCODE_TCFS_PATH_ERROR,            
    TCFS_ERRCODE_APPID_VERIFY_FAILED,            
    TCFS_ERRCODE_ADDRESS_VERIFY_FAILED,            
    TCFS_ERRCODE_NO_ENOUGH_SPACE,            
    TCFS_ERRCODE_SIGN_FAILED,    
    TCFS_ERRCODE_EXCCED_USER_MEM,        
    TCFS_ERRCODE_ACCESS_SERVER_FAILED,    
    TCFS_ERRCODE_ACCESS_FILE_FAILED,        
    TCFS_ERRCODE_EXCCED_MAX_FILE_NUM,        
};


#ifndef MAX_PATH_LEN
#define MAX_PATH_LEN 256
#endif

#define HASH_LEN 40

struct TreeNodeT
{
    char path[MAX_PATH_LEN + 1];
    char hash[HASH_LEN + 1];    
    uint64_t ctime;
    uint64_t size;
    int type;
};

struct HistoryVersionT
{
    char versionHash[HASH_LEN + 1];
    uint64_t ctime;    
    char rootHash[HASH_LEN + 1];
    uint64_t stage;
    char preVersion[HASH_LEN + 1];
};


#define APPID_LENGTH  128
#define APP_SIGN_LENGTH 64
#define ADDRESS_LENGTH 64
#define USER_SIGN_LENGTH 256

struct TCFSAppCertification {
    char appid[APPID_LENGTH];
    uint64_t timestamp;
    char appSign[APP_SIGN_LENGTH];
};

struct TCFSUserCertification {
    char address[ADDRESS_LENGTH];
    uint64_t timestamp;
    char userSign[USER_SIGN_LENGTH];
};


int TCFS_Init();

int TCFS_Uninit();

int TCFS_GetVersion(char *version, int len);

int TCFS_Upload(TCFSAppCertification *appCert, TCFSUserCertification *userCert, const char* srcPath, const char* tcfsPath, bool isDir);

int TCFS_Download(TCFSAppCertification *appCert, TCFSUserCertification *userCert, const char *dstPath, const char *tcfsPath, bool isDir);

int TCFS_Delete(TCFSAppCertification *appCert, TCFSUserCertification *userCert, const char *tcfsPath, int isDir);

int TCFS_Copy(TCFSAppCertification *appCert, TCFSUserCertification *userCert, const char *signTcfsPath, const char *tcfsPath);


int TCFS_ListTree(TCFSAppCertification *appCert, TCFSUserCertification *userCert, const char *tcfsPath, TreeNodeT node[], int maxNodeNum,  int *realNodeNum);

int TCFS_ListDir(TCFSAppCertification *appCert, TCFSUserCertification *userCert, const char *tcfsPath, int start, int len, TreeNodeT node[], int maxNodeNum,  int *realNodeNum);

int TCFS_GetSpace(TCFSAppCertification *appCert, TCFSUserCertification *userCert, uint64_t *totalSize, uint64_t *usedSize);

int TCFS_UserSign(const char *keyjson, const char *password, const char *data, char *signData, int signLen);

#ifdef __cplusplus
}
#endif


#endif

